var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
import { useSlots as _useSlots, defineComponent as _defineComponent } from "vue";
import { watch, ref, computed, onBeforeUnmount, render, h, Fragment, getCurrentInstance } from "vue";
import { vLoading } from "element-plus/es/index.mjs";
import { useResizeObserver } from "@vueuse/core";
import { bpmnViewerProps } from "./bpmnViewer.mjs";
import { parseXML, getFlowNodes, isFlowNode, isSequenceFlow } from "./helper.mjs";
import { CLICKABLE_MARKER, NONCLICKABLE_MARKER, TOOLTIP_CLASSNAME, STATE_OVERLAY_CLASSNAME } from "./constants.mjs";
import outlineProvider from "bpmn-js/lib/features/outline";
import { createNamespace } from "../../utils/index.mjs";
const name = createNamespace("bpmn-viewer");
const __default__ = {
  name
};
const __vue_sfc__ = /* @__PURE__ */ _defineComponent(__spreadProps(__spreadValues({}, __default__), {
  props: bpmnViewerProps,
  emits: ["element-click"],
  setup(__props, { expose: __expose, emit: __emit }) {
    var _a;
    __expose();
    const props = __props;
    const emit = __emit;
    const slots = _useSlots();
    const appContext = ((_a = getCurrentInstance()) == null ? void 0 : _a.appContext) || null;
    const container = ref();
    const viewer = ref();
    const diagramModel = ref();
    const tooltipOverlayIds = {};
    const stateOverlayIds = {};
    let clickableElements = [];
    useResizeObserver(container, () => {
      if (viewer.value) {
        resize();
      }
    });
    const defaultClickableElements = computed(() => {
      var _a2;
      return getFlowNodes((_a2 = diagramModel.value) == null ? void 0 : _a2.elementsById);
    });
    watch([() => props.viewerConstructor, container], () => {
      if (viewer.value) {
        viewer.value.destroy();
        viewer.value = null;
      }
      if (props.viewerConstructor && container.value) {
        viewer.value = new props.viewerConstructor({
          container: container.value,
          additionalModules: [outlineProvider],
          bpmnRenderer: {
            defaultFillColor: "var(--pd-bpmn-viewer-element-fill-color)",
            defaultStrokeColor: "var(--pd-bpmn-viewer-element-stroke-color)",
            defaultLabelColor: "var(--pd-bpmn-viewer-label-color)"
          }
        });
        const eventBus = viewer.value.get("eventBus");
        eventBus.on("element.click", (e) => {
          if (props.clickable && isElementClickable(e.element.id)) {
            emit("element-click", e.element);
          }
        });
        eventBus.on("element.hover", (e) => {
          if (e.element.waypoints) {
            const [{ x: x1 }, { x: x2 }] = e.element.waypoints;
            createTooltip(e.element, {
              left: (x2 - x1) / 2,
              bottom: -10
            });
          } else {
            createTooltip(e.element, {
              left: e.element.width / 2,
              bottom: -10
            });
          }
        });
        eventBus.on("element.out", (e) => {
          removeTooltip(e.element.id);
        });
      }
    });
    watch([() => props.xml, viewer], () => __async(this, null, function* () {
      if (props.xml && viewer.value) {
        yield viewer.value.importXML(props.xml);
        diagramModel.value = yield parseXML(props.xml);
        updateClickableNodes();
        updateStatus();
        resize();
      }
    }));
    watch([() => props.clickable, () => props.isElementClickable], () => {
      updateClickableNodes();
    });
    watch(() => props.active, () => updateStatus("active"), {
      deep: true
    });
    watch(() => props.incident, () => updateStatus("incident"), {
      deep: true
    });
    watch(() => props.canceled, () => updateStatus("canceled"), {
      deep: true
    });
    watch(() => props.completed, () => updateStatus("completed"), {
      deep: true
    });
    watch(() => props.highlight, () => updateStatus("highlight"), {
      deep: true
    });
    const isElementClickable = (eleId) => {
      return clickableElements.some(({ id }) => id === eleId);
    };
    const updateStatus = (type) => {
      if (viewer.value) {
        const injector = viewer.value.get("injector");
        const elementRegistry = injector.get("elementRegistry");
        const elements = elementRegistry.getAll();
        Object.values(elements).forEach((el) => {
          var _a2;
          if (!type || type === "highlight") {
            const isHighlightOrColor = (_a2 = props.highlight) == null ? void 0 : _a2[el.id];
            if (isHighlightOrColor) {
              if (typeof isHighlightOrColor === "string") {
                highlightElement(el.id, isHighlightOrColor);
              } else {
                highlightElement(el.id);
              }
            }
          }
          ["active", "completed", "canceled", "incident"].forEach((stateType) => {
            var _a3, _b;
            if (!type || type === stateType) {
              const isShowOrCount = (_a3 = props[stateType]) == null ? void 0 : _a3[el.id];
              if (isShowOrCount) {
                if (typeof isShowOrCount === "number") {
                  createStateOverlay(el.id, stateType, isShowOrCount);
                } else {
                  createStateOverlay(el.id, stateType);
                }
              } else if ((_b = stateOverlayIds[el.id]) == null ? void 0 : _b[stateType]) {
                removeOverlays(stateOverlayIds[el.id][stateType]);
                delete stateOverlayIds[el.id][stateType];
              }
            }
          });
        });
      }
    };
    const createTooltip = (ele, position) => {
      var _a2, _b;
      const tooltipSlot = slots[`${ele.id}-tooltip`];
      if (((_a2 = props.tooltip) == null ? void 0 : _a2[ele.id]) || tooltipSlot) {
        const div = document.createElement("div");
        div.className = TOOLTIP_CLASSNAME;
        const vm = h(Fragment, (tooltipSlot == null ? void 0 : tooltipSlot({ element: ele })) || [(_b = props.tooltip) == null ? void 0 : _b[ele.id]]);
        vm.appContext = appContext;
        render(vm, div);
        const id = addOverlays(ele.id, "tooltip", {
          position,
          html: div
        });
        if (id)
          tooltipOverlayIds[ele.id] = id;
      }
    };
    const removeTooltip = (eleId) => {
      if (tooltipOverlayIds[eleId])
        removeOverlays(tooltipOverlayIds[eleId]);
    };
    const createStateOverlay = (eleId, stateType, count) => {
      var _a2, _b;
      if ((_a2 = props[stateType]) == null ? void 0 : _a2[eleId]) {
        const svg = {
          active: '<svg focusable="false" fill="currentColor" width="16" height="16" viewBox="0 0 32 32" aria-hidden="true" xmlns="http://www.w3.org/2000/svg"><path d="M16,2A14,14,0,1,0,30,16,14,14,0,0,0,16,2Zm0,26A12,12,0,1,1,28,16,12,12,0,0,1,16,28Z"></path><path d="M16,10a6,6,0,1,0,6,6A6,6,0,0,0,16,10Z"></path></svg>',
          incident: '<svg focusable="false" fill="currentColor" width="16" height="16" viewBox="0 0 16 16" aria-hidden="true" xmlns="http://www.w3.org/2000/svg"><path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2 c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z"></path><path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8 c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" opacity="0"></path></svg>',
          canceled: '<svg focusable="false" fill="currentColor" width="16" height="16" viewBox="0 0 32 32" aria-hidden="true" xmlns="http://www.w3.org/2000/svg"><path d="M2,16H2A14,14,0,1,0,16,2,14,14,0,0,0,2,16Zm23.15,7.75L8.25,6.85a12,12,0,0,1,16.9,16.9ZM8.24,25.16A12,12,0,0,1,6.84,8.27L23.73,25.16a12,12,0,0,1-15.49,0Z"></path></svg>',
          completed: '<svg focusable="false" fill="currentColor" width="16" height="16" viewBox="0 0 32 32" aria-hidden="true" xmlns="http://www.w3.org/2000/svg"><path d="M14 21.414L9 16.413 10.413 15 14 18.586 21.585 11 23 12.415 14 21.414z"></path><path d="M16,2A14,14,0,1,0,30,16,14,14,0,0,0,16,2Zm0,26A12,12,0,1,1,28,16,12,12,0,0,1,16,28Z"></path></svg>'
        };
        const div = document.createElement("div");
        div.className = `${STATE_OVERLAY_CLASSNAME} ${STATE_OVERLAY_CLASSNAME}--${stateType}`;
        div.innerHTML = svg[stateType];
        if (count) {
          div.innerHTML += `<span class="${STATE_OVERLAY_CLASSNAME}__count">${count}</span>`;
        }
        if ((_b = stateOverlayIds[eleId]) == null ? void 0 : _b[stateType]) {
          removeOverlays(stateOverlayIds[eleId][stateType]);
          delete stateOverlayIds[eleId][stateType];
        }
        const position = {
          active: {
            left: 0,
            bottom: 0
          },
          incident: {
            right: 0,
            bottom: 0
          },
          canceled: {
            left: 0,
            top: 0
          },
          completed: {
            right: 0,
            top: 0
          }
        };
        const id = addOverlays(eleId, `${stateType}-state`, {
          position: position[stateType],
          html: div
        });
        if (id) {
          if (!stateOverlayIds[eleId]) {
            stateOverlayIds[eleId] = {};
          }
          stateOverlayIds[eleId][stateType] = id;
        }
      }
    };
    const updateClickableNodes = () => {
      if (viewer.value) {
        const injector = viewer.value.get("injector");
        const elementRegistry = injector.get("elementRegistry");
        const elements = elementRegistry.getAll();
        clickableElements = props.isElementClickable ? Object.values(elements).filter((ele) => props.isElementClickable(ele)) : defaultClickableElements.value;
        Object.values(elements).forEach((ele) => {
          if (!props.clickable) {
            removeMarker(ele.id, CLICKABLE_MARKER);
            removeMarker(ele.id, NONCLICKABLE_MARKER);
          } else if (isElementClickable(ele.id)) {
            addMarker(ele.id, CLICKABLE_MARKER);
            removeMarker(ele.id, NONCLICKABLE_MARKER);
          } else if (ele.$type !== "label" && isFlowNode(ele)) {
            addMarker(ele.id, NONCLICKABLE_MARKER);
          }
        });
      }
    };
    const resize = () => {
      if (viewer.value) {
        const canvas = viewer.value.get("canvas");
        canvas.zoom("fit-viewport", "auto");
      }
    };
    const removeMarker = (eleId, marker) => {
      if (viewer.value) {
        const canvas = viewer.value.get("canvas");
        canvas.removeMarker(eleId, marker);
      }
    };
    const addMarker = (eleId, marker) => {
      if (viewer.value) {
        const canvas = viewer.value.get("canvas");
        canvas.addMarker(eleId, marker);
      }
    };
    const addOverlays = (element, type, overlay) => {
      if (viewer.value) {
        const overlays = viewer.value.get("overlays");
        return overlays.add(element, type, overlay);
      }
    };
    const removeOverlays = (id) => {
      if (viewer.value) {
        const overlays = viewer.value.get("overlays");
        overlays.remove(id);
      }
    };
    const highlightElement = (id, color = "var(--pd-bpmn-viewer-highlight-color)") => {
      if (viewer.value) {
        const injector = viewer.value.get("injector");
        const elementRegistry = injector.get("elementRegistry");
        const element = elementRegistry.get(id);
        if (element) {
          const gfx = elementRegistry.getGraphics(element);
          const graphicsFactory = injector.get("graphicsFactory");
          if (isSequenceFlow(element.businessObject)) {
            const { di } = element;
            di.set("stroke", color);
            di.set("fill", color);
            graphicsFactory.update("connection", element, gfx);
          } else {
            const { di } = element;
            di.set("stroke", color);
            graphicsFactory.update("shape", element, gfx);
          }
        }
      }
    };
    onBeforeUnmount(() => {
      var _a2;
      (_a2 = viewer.value) == null ? void 0 : _a2.destroy();
    });
    const __returned__ = { name, props, emit, slots, appContext, container, viewer, diagramModel, tooltipOverlayIds, stateOverlayIds, get clickableElements() {
      return clickableElements;
    }, set clickableElements(v) {
      clickableElements = v;
    }, defaultClickableElements, isElementClickable, updateStatus, createTooltip, removeTooltip, createStateOverlay, updateClickableNodes, resize, removeMarker, addMarker, addOverlays, removeOverlays, highlightElement, get vLoading() {
      return vLoading;
    } };
    Object.defineProperty(__returned__, "__isScriptSetup", { enumerable: false, value: true });
    return __returned__;
  }
}));
import { renderSlot as _renderSlot, toDisplayString as _toDisplayString, createElementVNode as _createElementVNode, openBlock as _openBlock, createElementBlock as _createElementBlock, createCommentVNode as _createCommentVNode, withDirectives as _withDirectives } from "vue";
const _hoisted_1 = {
  class: "pagoda-bpmn-viewer",
  ref: "container"
};
const _hoisted_2 = {
  key: 0,
  class: "pagoda-bpmn-viewer__empty-text-wrapper"
};
const _hoisted_3 = { class: "pagoda-bpmn-viewer__empty-text" };
function __vue_render__(_ctx, _cache, $props, $setup, $data, $options) {
  return _withDirectives((_openBlock(), _createElementBlock("div", _hoisted_1, [
    !_ctx.xml && !_ctx.loading ? (_openBlock(), _createElementBlock("div", _hoisted_2, [
      _renderSlot(_ctx.$slots, "empty", {}, () => [
        _createElementVNode(
          "div",
          _hoisted_3,
          _toDisplayString(_ctx.emptyText),
          1
          /* TEXT */
        )
      ])
    ])) : _createCommentVNode("v-if", true)
  ])), [
    [$setup["vLoading"], _ctx.loading]
  ]);
}
__vue_sfc__.render = __vue_render__;
var stdin_default = __vue_sfc__;
export {
  stdin_default as default
};
