var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
import { ref, nextTick } from "vue";
import Sortable from "sortablejs";
import { removeClass, getMapFromTree } from "pagoda-shared";
function removeTableHoverClass(elTableEl) {
  Array.from(elTableEl.querySelectorAll(".el-table__row")).forEach(
    (tr) => {
      removeClass(tr, "hover-row");
    }
  );
}
function useElTableSort({
  rowKey,
  cancelMove,
  onEnd
}) {
  const draging = ref(false);
  const dragingItemKey = ref(null);
  const dataIdAttr = "data-sortable-key";
  let sortable;
  const initTableSort = ({
    elTableEl,
    elTableData
  }) => {
    sortable == null ? void 0 : sortable.destroy();
    const tbody = elTableEl.querySelector(
      ".el-table__body-wrapper tbody"
    );
    const fixedTbody = elTableEl.querySelector(
      ".el-table__fixed tbody"
    );
    const fixedRightTbody = elTableEl.querySelector(
      ".el-table__fixed-right tbody"
    );
    const tableDataMap = getMapFromTree(elTableData, rowKey).map;
    const tbodyTrs = Array.from(tbody.querySelectorAll("tr"));
    const fixedTbodyTrs = fixedTbody ? Array.from(fixedTbody.querySelectorAll("tr")) : [];
    const fixedRightTbodyTrs = fixedRightTbody ? Array.from(fixedRightTbody.querySelectorAll("tr")) : [];
    tbodyTrs.forEach((tr, idx) => {
      const rowKeyValue = elTableData[idx][rowKey];
      tr.setAttribute(dataIdAttr, rowKeyValue);
      if (fixedTbody)
        fixedTbodyTrs[idx].setAttribute(dataIdAttr, rowKeyValue);
      if (fixedRightTbody)
        fixedRightTbodyTrs[idx].setAttribute(dataIdAttr, rowKeyValue);
    });
    const fixedSortable = fixedTbody && Sortable.create(fixedTbody, {
      dataIdAttr,
      disabled: true
    });
    const fixedRightSortable = fixedRightTbody && Sortable.create(fixedRightTbody, {
      dataIdAttr,
      disabled: true
    });
    sortable = Sortable.create(tbody, {
      dataIdAttr,
      setData(dataTransfer) {
        dataTransfer.setData("Text", "");
      },
      onStart: ({ oldIndex }) => {
        draging.value = true;
        dragingItemKey.value = elTableData[oldIndex][rowKey];
        removeTableHoverClass(elTableEl);
      },
      onMove(event) {
        const relatedRowKey = event.related.getAttribute(dataIdAttr);
        const draggedRowKey = event.dragged.getAttribute(dataIdAttr);
        if (cancelMove == null ? void 0 : cancelMove(tableDataMap[draggedRowKey], tableDataMap[relatedRowKey]))
          return false;
      },
      onChange: () => {
        if (fixedSortable || fixedRightSortable) {
          const order = sortable.toArray();
          fixedSortable == null ? void 0 : fixedSortable.sort(order);
          fixedRightSortable == null ? void 0 : fixedRightSortable.sort(order);
        }
      },
      onEnd: (_0) => __async(this, [_0], function* ({ newIndex, oldIndex }) {
        const dragRow = elTableData[oldIndex];
        const relativeRow = elTableData[newIndex];
        elTableData.splice(oldIndex, 1);
        elTableData.splice(newIndex, 0, dragRow);
        draging.value = false;
        dragingItemKey.value = null;
        yield nextTick();
        removeTableHoverClass(elTableEl);
        onEnd(dragRow, relativeRow);
      })
    });
  };
  const sortCellClasses = ({ row }) => {
    return draging.value ? dragingItemKey.value === row[rowKey] ? "sortable-draging-cell" : "sortable-cell" : "";
  };
  return {
    sortCellClasses,
    initTableSort
  };
}
export {
  removeTableHoverClass,
  useElTableSort
};
