var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var stdin_exports = {};
__export(stdin_exports, {
  useTabs: () => useTabs
});
module.exports = __toCommonJS(stdin_exports);
var import_vue = require("vue");
var import_error = require("../../../utils/error");
var import_lodash_unified = require("lodash-unified");
function useTabs({
  activeTab,
  defaultTabs = [],
  beforeTabsRemove,
  onActiveTabChange
}) {
  const currentTabName = (0, import_vue.ref)("");
  const tabs = (0, import_vue.ref)([...defaultTabs]);
  (0, import_vue.watch)(
    activeTab,
    (activeTabConfig) => {
      if (activeTabConfig) {
        if (!activeTabConfig.name) {
          throw new import_error.PagodaUiError(
            "Component Layout",
            "activeTab \u5C5E\u6027\u7F3A\u5C11\u552F\u4E00\u6807\u8BC6 name \u5B57\u6BB5"
          );
        }
        const tabIndex = tabs.value.findIndex(
          (tab) => tab.name === activeTabConfig.name
        );
        const newTabConfig = Object.assign({}, activeTabConfig);
        if (tabIndex === -1) {
          tabs.value.push(newTabConfig);
        } else {
          tabs.value[tabIndex] = newTabConfig;
        }
        currentTabName.value = activeTabConfig.name;
      } else {
        currentTabName.value = "";
      }
    },
    {
      immediate: true
    }
  );
  function isTabClosable(tab) {
    return "closable" in tab && (0, import_lodash_unified.isBoolean)(tab.closable) ? tab.closable : true;
  }
  function hasTab(tabName) {
    return tabs.value.some((tab) => tab.name === tabName);
  }
  function selectTab(tab) {
    const tabName = tab.props.name;
    if (tabName !== currentTabName.value) {
      const selectedTab = tabs.value.find(
        (tabConfig) => tabConfig.name === tabName
      );
      if (selectedTab)
        onActiveTabChange == null ? void 0 : onActiveTabChange(selectedTab);
    }
  }
  function closeActiveTab() {
    if (activeTab.value) {
      closeTab(activeTab.value.name);
    }
  }
  function closeTab(removeTabName, safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab) && tab.name === removeTabName;
    const next = () => {
      tabs.value = tabs.value.filter((tab, index) => {
        if (activeTab.value && activeTab.value.name === removeTabName && tab.name === removeTabName && !safeClose) {
          const nextActiveTab = tabs.value[index + 1] || tabs.value[index - 1];
          if (nextActiveTab) {
            onActiveTabChange == null ? void 0 : onActiveTabChange(nextActiveTab);
          }
        }
        return !isTabShouldClose(tab);
      });
    };
    if (beforeTabsRemove) {
      const removeTabConfig = tabs.value.find(
        (tab) => tab.name === removeTabName
      );
      if (removeTabConfig)
        beforeTabsRemove([removeTabConfig], next, safeClose);
    } else {
      next();
    }
  }
  function closeAllTabs(safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab);
    const next = () => {
      tabs.value = tabs.value.filter((tab) => {
        return !isTabShouldClose(tab);
      });
      if (tabs.value.length && !safeClose && activeTab.value && !hasTab(activeTab.value.name)) {
        onActiveTabChange == null ? void 0 : onActiveTabChange(tabs.value[tabs.value.length - 1]);
      }
    };
    if (beforeTabsRemove) {
      const canCloseAllTabs = tabs.value.filter((tab) => {
        return isTabShouldClose(tab);
      });
      beforeTabsRemove(canCloseAllTabs, next, safeClose);
    } else {
      next();
    }
  }
  function closeOthersTabs(ignoreTabName, safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab) && tab.name !== ignoreTabName;
    const next = () => {
      tabs.value = tabs.value.filter((tab) => {
        return !isTabShouldClose(tab);
      });
      if (tabs.value.length && !safeClose && activeTab.value && !hasTab(activeTab.value.name)) {
        onActiveTabChange == null ? void 0 : onActiveTabChange(tabs.value[tabs.value.length - 1]);
      }
    };
    if (beforeTabsRemove) {
      const othersCanCloseTabs = tabs.value.filter((tab) => {
        return isTabShouldClose(tab);
      });
      beforeTabsRemove(othersCanCloseTabs, next, safeClose);
    } else {
      next();
    }
  }
  return {
    currentTabName,
    tabs,
    isTabClosable,
    selectTab,
    closeActiveTab,
    closeTab,
    closeAllTabs,
    closeOthersTabs
  };
}
