var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var stdin_exports = {};
__export(stdin_exports, {
  useTableStore: () => useTableStore
});
module.exports = __toCommonJS(stdin_exports);
var import_vue = require("vue");
var import_lodash_unified = require("lodash-unified");
var import_pagoda_shared = require("pagoda-shared");
function useTableStore(options) {
  const {
    data,
    getColumns = () => [],
    rowStatesCreator,
    getCellsStatesCreator,
    childrenKey = "children"
  } = options;
  const store = {
    states: /* @__PURE__ */ new WeakMap(),
    tableCacheData: /* @__PURE__ */ new WeakMap()
  };
  if (data) {
    addStore(data);
  }
  function resetStore(data2) {
    store.states = /* @__PURE__ */ new WeakMap();
    store.tableCacheData = /* @__PURE__ */ new WeakMap();
    addStore(data2);
  }
  function getRowsCache(rows) {
    return rows.map((row) => {
      return store.tableCacheData.get(row);
    });
  }
  function setRowsCache(rows) {
    rows.forEach((row) => {
      store.tableCacheData.set(row, (0, import_lodash_unified.cloneDeep)(row));
    });
  }
  function deleteRowsCache(rows) {
    rows.forEach((row) => {
      store.tableCacheData.delete(row);
    });
  }
  function addStore(data2, parent) {
    (0, import_pagoda_shared.loopTree)(
      data2,
      childrenKey,
      (row, parent2) => {
        const rowConfig = {
          states: {},
          cellsStates: {}
        };
        if (rowStatesCreator) {
          rowConfig.states = (0, import_vue.reactive)(rowStatesCreator(row, parent2));
        }
        const cellsStatesCreator = getCellsStatesCreator == null ? void 0 : getCellsStatesCreator();
        if (cellsStatesCreator) {
          getColumns().forEach((prop) => {
            const cellStatesCreator = cellsStatesCreator[prop];
            if (cellStatesCreator) {
              rowConfig.cellsStates[prop] = (0, import_vue.reactive)(
                cellStatesCreator(row, parent2)
              );
            }
          });
        }
        store.states.set(row, rowConfig);
        setRowsCache([row]);
      },
      parent
    );
  }
  function resetCellsStates(data2) {
    if (!getCellsStatesCreator)
      return;
    const columns = getColumns();
    const cellsStatesCreator = getCellsStatesCreator();
    (0, import_pagoda_shared.loopTree)(data2, childrenKey, (row, parent) => {
      const rowConfig = store.states.get(row);
      if (!rowConfig)
        return;
      Object.keys(rowConfig.cellsStates).forEach((prop) => {
        if (!columns.includes(prop)) {
          delete rowConfig.cellsStates[prop];
        }
      });
      columns.forEach((prop) => {
        const cellStatesCreator = cellsStatesCreator[prop];
        if (cellStatesCreator && !rowConfig.cellsStates[prop]) {
          rowConfig.cellsStates[prop] = (0, import_vue.reactive)(
            cellStatesCreator(row, parent)
          );
        }
      });
    });
  }
  function getRowStates(row) {
    var _a;
    return (_a = store.states.get(row)) == null ? void 0 : _a.states;
  }
  function getCellsStates(row) {
    var _a;
    return (_a = store.states.get(row)) == null ? void 0 : _a.cellsStates;
  }
  function getCellStates(row, cell) {
    var _a, _b;
    return (_b = (_a = store.states.get(row)) == null ? void 0 : _a.cellsStates) == null ? void 0 : _b[cell];
  }
  function setRowsStates(rows, states) {
    (0, import_pagoda_shared.loopTree)(rows, childrenKey, (row) => {
      const rowStates = getRowStates(row);
      if (rowStates) {
        Object.assign(rowStates, states);
      }
    });
  }
  function setCellStates(rows, cells, states) {
    (0, import_pagoda_shared.loopTree)(rows, childrenKey, (row) => {
      const cellsStates = getCellsStates(row);
      cells.forEach((cell) => {
        if (cellsStates == null ? void 0 : cellsStates[cell]) {
          Object.assign(cellsStates[cell], states);
        }
      });
    });
  }
  function delRowsStates(rows) {
    (0, import_pagoda_shared.loopTree)(rows, childrenKey, (row) => {
      store.states.delete(row);
    });
  }
  function delRowsStore(rows) {
    (0, import_pagoda_shared.loopTree)(rows, childrenKey, (row) => {
      store.states.delete(row);
      store.tableCacheData.delete(row);
    });
  }
  return Object.assign(store, {
    addStore,
    resetStore,
    setRowsStates,
    setCellStates,
    resetCellsStates,
    setRowsCache,
    getRowsCache,
    getCellStates,
    getRowStates,
    getCellsStates,
    delRowsStates,
    deleteRowsCache,
    delRowsStore
  });
}
